//
// fingerprint.rs
// Copyright (C) 2025 Óscar García Amor <ogarcia@connectical.com>
// Distributed under terms of the GNU GPLv3 license.
//

use std::fmt::Write;
use std::time::SystemTime;

const ICONS: [&str; 46] = [
    "hashtag",
    "heart",
    "hotel",
    "university",
    "plug",
    "ambulance",
    "bus",
    "car",
    "plane",
    "rocket",
    "ship",
    "subway",
    "truck",
    "yen",
    "euro",
    "btc",
    "dollar",
    "sterling",
    "archive",
    "chart",
    "bed",
    "beer",
    "bell",
    "binoculars",
    "cake",
    "bomb",
    "briefcase",
    "bug",
    "camera",
    "cart",
    "certificate",
    "mug",
    "cloud",
    "mug",
    "comment",
    "cube",
    "utensils",
    "database",
    "diamond",
    "exclamation",
    "eye",
    "flag",
    "flask",
    "futbol",
    "gamepad",
    "cap"
];

// Retrieves an icon based on the provided SHA256 hash (sourced from RustedLessPass)
fn get_icon(sha256: &str) -> &'static str {
    // Convert the hexadecimal SHA256 hash to a u32 integer
    let sum = match u32::from_str_radix(sha256, 16) {
        Ok(parsed_value) => parsed_value,
        Err(_) => return "rust",
    };
    // Calculate the index of the icon based on the hash value
    let index = sum % ICONS.len() as u32;
    ICONS[index as usize]
}

// Calculates and returns a fingerprint based on the input string (sourced from RustedLessPass)
pub fn calculate(input: &str) -> Vec<String> {
    let mut hashed_input_icons: Vec<String> = Vec::new();

    if input.is_empty() {
        hashed_input_icons.push("heart".to_string());
        hashed_input_icons.push("rust".to_string());
        hashed_input_icons.push("rocket".to_string());
    } else {
        // Calculate the SHA256 fingerprint of the input string
        let hashed_input: String = lesspass::get_fingerprint(input)
            // Convert the byte array to a hexadecimal string
            .iter()
            .fold(String::new(), |mut acc, &byte| {
                write!(acc, "{:02x}", byte).expect("Failed to write to string");
                acc
            });

        // Divide the hashed input into segments and assign icons
        let mut x = 0;
        let mut y = 6;
        for _i in 0..3 {
            // Get an icon corresponding to each segment of the hashed input
            let hashed_segment_icon = get_icon(&hashed_input[x..y]);
            hashed_input_icons.push(hashed_segment_icon.to_string());
            x += 6;
            y += 6;
        }
    }
    // Return the vector of hashed input icons
    hashed_input_icons
}

// Returns a pseudoramdom vector of icons
pub fn calculate_fake() -> Vec<String> {
    let mut icons: Vec<String> = Vec::new();
    for _i in 0..3 {
        let index = SystemTime::now()
            .duration_since(SystemTime::UNIX_EPOCH)
            .map(|d| d.as_nanos())
            .unwrap_or(42) as usize % ICONS.len();
        icons.push(ICONS[index as usize].to_string());
    }
    icons
}

